/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

### File Specific:
   main-programmer: hdavid, amaechler
*/

#include "lightening_effect.h"

#include "state.h"
#include "util/loading/load_param.h"
#include "util/loading/factory.h"
#include "effects/billboard.h"

#include "glincl.h"
#include "parser/tinyxml/tinyxml.h"

#include "shell_command.h"

SHELL_COMMAND(activate, LighteningEffect, activateLightening);
SHELL_COMMAND(deactivate, LighteningEffect, deactivateLightening);

using namespace std;

CREATE_FACTORY(LighteningEffect, CL_LIGHTENING_EFFECT);

LighteningEffect::LighteningEffect(const TiXmlElement* root)
{
	this->setClassID(CL_LIGHTENING_EFFECT, "LighteningEffect");
  
	this->init();

	if (root != NULL)
		this->loadParams(root);

  if(this->lighteningActivate)
	 this->activate();
}

LighteningEffect::~LighteningEffect()
{
	this->deactivate();
}

void LighteningEffect::loadParams(const TiXmlElement* root)
{
	WeatherEffect::loadParams(root);
  
  LoadParam(root, "coord", this, LighteningEffect, coord);
  LoadParam(root, "option", this, LighteningEffect, setLighteningOption);
}


bool LighteningEffect::init()
{
  lighteningActivate = false;
  
  this->time = 0.0;
  this->flashFrequency = 0.6f;
  this->flashConstTime = 0.07f;

  this->width = 40.0f;
  this->height = 10.0f;
  this->bNewCoordinate = false;

  this->seedX = 10.f;
  this->seedZ = 10.0f;
  this->seedTime = 4.0f;

  this->billboard = new Billboard(NULL);
  this->billboard->setTexture("maps/lightning_bolt.png");
  this->billboard->setSize(this->width, this->height);
  this->billboard->setAbsCoor(0.0f, 50.0f, 0.0f);
  this->billboard->setVisibiliy(false);

/*
  this->soundSource = NULL;
  this->thunderBuffer = NULL;

  this->soundSource.setSourceNode(this);

  //load sound
  if (this->thunderBuffer != NULL)
    ResourceManager::getInstance()->unload(this->thunderBuffer);
  this->thunderBuffer = (OrxSound::SoundBuffer*)ResourceManager::getInstance()->load("sound/thunder.wav", WAV);*/
}

void LighteningEffect::coord(float x, float y, float z)
{
  this->billboard->setAbsCoor(x, y, z);
  this->mainPosX = x;
  this->mainPosY = y;
  this->mainPosZ = z;
  
}

bool LighteningEffect::activate()
{
	PRINTF(0)( "Activating LighteningEffect\n" );
  this->billboard->setVisibiliy(true);
  lighteningActivate = true;
}


bool LighteningEffect::deactivate()
{
	PRINTF(0)("Deactivating LighteningEffect\n");
  this->billboard->setVisibiliy(false);
  lighteningActivate = false;
}

void LighteningEffect::tick (float dt)
{
  if(!lighteningActivate)
    return;
  
  this->time += dt;

  if( this->time > this->flashFrequency)
  {
    this->billboard->setVisibiliy(true);
    this->time = 0.0f;
    //this->soundSource.play(this->thunderBuffer);
  }
  else if( this->billboard->isVisible() && this->time > this->flashConstTime)
  {
    this->billboard->setVisibiliy(false);
    this->time = 0.0f;
    this->bNewCoordinate = true;
  }

  if( this->bNewCoordinate)
  {
    this->flashFrequency = this->seedTime * (float)rand()/(float)RAND_MAX + 0.1;
    this->billboard->setAbsCoor(this->mainPosX - this->seedX * (float)rand()/(float)RAND_MAX, this->mainPosY, this->mainPosZ + this->seedZ * (float)rand()/(float)RAND_MAX);
    this->bNewCoordinate = false;
  }
}
